#! /bin/bash
# To completely install Tunnel on RHEL OS
TUNNEL_LOG_PATH=/opt/dell/srvadmin/var/log
TUNNEL_LOG=/opt/dell/srvadmin/var/log/itp_log.txt

### COMMON  functions ###
usage() 
{
    echo "usage: $0 [-h] [-i] [-u] [-p <path to iTP folder>]"
}

# logging
msg()
{
	#echo -e "`date`:\t$1"
	echo -e "`date`:\t$1" >> $TUNNEL_LOG
	
	if [ "$2" == 1 ]
	then
		echo $1
		echo "cannot proceed... exiting installation.See $TUNNEL_LOG for messages"
		exit 1 # stop the script
	fi
}

#find the current OS
getOS()
{
#	values of OS for each type
#	0:other
#	1:RHEL3
#	2:RHEL4
#	3:RHEL5
#	4:SLES9
#	5:SLES10
#	6:SLES11
#	7:ESX 3.5
#	8:ESX 4

    # Else check if operating system is ESX.
    if [ -f /etc/vmware-release ]; then
        VER4=`vmware -v | cut -d ' ' -f3 | cut -d '.' -f 1`
        VER3=`vmware -v | cut -d ' ' -f4 | cut -d '.' -f 1`
        # Check if operating system is ESX 3.5.
        if [ "${VER3}" = "3" ]; then
			export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/var/pegasus/lib
			export PATH=$PATH:/var/pegasus/bin
			return 7;
        # Check if operating system is ESX4.
        elif [ "${VER4}" = "4" ]; then
			return 8;
        fi
	# Check if operating system is RHEL3.
    elif [ -f /etc/redhat-release ] && [ `grep -c "Taroon" /etc/redhat-release` -gt 0 ]; then
		return 1;
    # Else check if operating system is RHEL4.
    elif [ -f /etc/redhat-release ] && [ `grep -c "Nahant" /etc/redhat-release` -gt 0 ]; then
		return 2;
    # Else check if operating system is RHEL5.
    elif [ -f /etc/redhat-release ] && [ `grep -c "Tikanga" /etc/redhat-release` -gt 0 ]; then
		return 3;
    # Else check if operating system is SLES.
    elif [ -f /etc/SuSE-release ]; then
        LOC_VERSION=`cat /etc/SuSE-release | grep "VERSION" | sed  -e 's#[^0-9]##g'`
        # Check if operating system is SLES9.
        if [ "${LOC_VERSION}" = "9" ]; then
			return 4;
        # Check if operating system is SLES10.
        elif [ "${LOC_VERSION}" = "10" ]; then
			return 5;
        # Check if operating system is SLES11.
        elif [ "${LOC_VERSION}" = "11" ]; then
			return 6;
        fi
    fi

	return 0
}

#find if architecture 32 or 64 bit running
getarchtype()
{
#get the architecture flag where installtion is done
	export ARCH=`arch`

	if [ "$ARCH" == "x86_64" ]; then
		return 1 #64 bit
	else
		return 0 #32 bit
	fi	
}

#does cimmof exist?
isCimmofAvailable()
{
which cimmof > /dev/null 2>&1

	if [ $? -eq 0 ]
	then
		msg "cimmof is installed"
	else
		msg "error:cimmof is not installed" 1
	fi
}
	
#does cimprovider exist?
isCimproviderAvailable()
{
which cimprovider > /dev/null 2>&1

	if [ $? -eq 0 ]
	then
		msg "cimprovider is installed"
	else
		msg "error:cimprovider is not installed" 1
	fi
}

#does wbemexec exist?
isWbemexecAvailable()
{
which wbemexec > /dev/null 2>&1

	if [ $? -eq 0 ]
	then
		msg "wbemexec is installed"
	else
		msg "error:wbemexec is not installed" 1
	fi
}

#are environmental variables set?
areEnvironVariableSet()
{
printenv | grep -i pegasus > temp.txt
PEGENV=`cat temp.txt 2> /dev/null`

	if [ -z "$PEGENV" ]
	then
		msg "Pegasus environmental variable not set"
	fi
}

#where does Pegasus repository exist?	
getPegasusRepository()
{
	PEGASUSREPOSITORY=`pegasus_repository`

	if [ $? -ne 0 ]
	then
		msg "pegasus repository not found" 1
	fi
}

#check if current OS is for pegasus or sfcb
#return 1 for pegasus cimom, 2 for sfcb cimom
isSfcbPegasusOS()
{
	
	#run appropriate functions for each OS/arch. type
	case $OS in
		# ESX3.5 will run pegasus cimom
		[7]) 
			return 1
		;;
		# RHEL 4,5, SLES 10,11 and ESX4 will run sfcb cimom
		[2,3,5,6,8]) 
			return 2
		;;
		#Unsupported OS
		[0])
			return 3
		;;
		*) msg "unsupported OS for SSA" 1;;
	esac
}

#get library path
getLibpath()
{
	isSfcbPegasusOS

	case $? in
		1) #pegasus cimom
			p="/var/pegasus/lib"
			if test ! -d $p
			then
				mkdir $p
			fi
		;;
		[2,3]) #sfcb cimom
			if [ $OS -eq 7 ] || [ $OS -eq 8 ]; then
				p="/lib/cmpi"
			else
				p="/usr/lib/cmpi"
			fi

			if test ! -d $p
			then
				mkdir $p
			fi
		;;
		*) #add /bin to /etc/ld.so.conf.d/omacim.conf
			msg "getLibpath:invalid OS" 1
		;;
	esac

	echo $p$1
	return 0
}

isTunnelAvailable()
{
	if cimprovider -l | grep -i DellOpenManageTunnelProviderModule > /dev/null 2>&1

	then
		return 1
	else
		return 0
	fi
}

### PEGASUS Functions ###
#install pegasus
install_pegasus()
{
	#does cimmof exist?
	isCimmofAvailable
		
	#does cimprovider exist?
	isCimproviderAvailable

	#does wbemexec exist?
	isWbemexecAvailable

	#are environmental variables set?
	areEnvironVariableSet

	#where does Pegasus repository exist?	
	getPegasusRepository

#Pegasus is running
	msg "starting cimserver..."
	cimserver > /dev/null 2>&1
	
	if [ $? -eq 0 ]
	then
		/etc/init.d/pegasus stop #/etc/init.d/tog-pegasus stop
		/etc/init.d/pegasus start #/etc/init.d/tog-pegasus start
		msg "pegasus restarted"
	else
		msg "cimserver running..."
	fi
		
#ensure tunnel is already not registered (cimprovider -l should not list DellOpenManageModule)
	msg "checking if tunnel is already running"
	isTunnelAvailable
		
	if [ $? -eq 1 ]
	then
		msg "tunnel registered, uninstalling it..."
		msg "deleting DellOpenManageTunnelProviderModule provider..."
        	cimprovider -r -m DellOpenManageTunnelProviderModule >/dev/null 2>&1

		isTunnelAvailable
			
		if [ $? -eq 0 ]
		then
			msg "tunnel provider successfully removed"
		else 
			msg "tunnel provider could not be removed"
		fi
		
		msg "removing symbolic link..."
        	TUNNEL_LIB_PATH=`getLibpath`
        	rm -f $TUNNEL_LIB_PATH/libcmpiTunnelProvider.so >/dev/null 2>&1

		msg "tunnel uninstallation complete"
	else
		msg "tunnel is not registered. Registering..."
	fi
	
	
#create a symbolic link to libomacim32/64.so.1.0.0 from the $PEGASUSREPOSITORY directory 
#link the shared tunnel library according to the bit flag
	PEG_LIB_PATH=`getLibpath`
	
	ls $INSTALLPATH/lib/libcmpiTunnelProvider.so.1.* > $INSTALLPATH/tunnel/temp.log
	while read path
	do
		ln -sf $path $PEG_LIB_PATH/libcmpiTunnelProvider.so
	done < $INSTALLPATH/tunnel/temp.log

	rm -f $INSTALLPATH/tunnel/temp.log	
#compile all mofs in dell_linux_tunnel.mof
	msg "compiling dell_linux_tunnel.mof..."
	cimmof -I $INSTALLPATH/etc/schema -aEV -n root/dcim/sysman $INSTALLPATH/etc/schema/dell_linux_tunnel.mof >> $TUNNEL_LOG 2>&1

	if [ $? -ne 0 ]
	then
		msg "error:could not compile dell_linux_tunnel.mof" 1
	fi

#compile all mofs in dell_linux_interop.mof
	msg "compiling dell_linux_interop.mof..."
	cimmof -I $INSTALLPATH/etc/schema -aEV -n root/interop $INSTALLPATH/etc/schema/dell_linux_tunnel_interop.mof >> $TUNNEL_LOG 2>&1

	if [ $? -ne 0 ]
	then
		msg "error:could not compile dell_linux_tunnel_interop.mof" 1
	fi

#Register omacim as a provider for Pegasus
	msg "registering tunnel as a provider for Pegasus..."
	cimmof -n root/PG_InterOp $INSTALLPATH/etc/schema/pegasus_tunnel_provider.mof  >> $TUNNEL_LOG 2>&1

	if [ $? -ne 0 ]
	then
		msg "error:could not register pegasus_tunnel_provider.mof" 1
	fi
	
	cimmof -n root/PG_InterOp $INSTALLPATH/etc/schema/pegasus_tunnel_interop_provider.mof >> $TUNNEL_LOG 2>&1
	
	if [ $? -ne 0 ]
	then
		msg "error:could not register pegasus_tunnel_interop_provider.mof" 1
	fi

#final check
	isTunnelAvailable
		
	if [ $? -eq 0 ]
	then
		msg "tunnel is not registered. Install unsuccessful"
		exit 0
	fi
	msg "tunnel is registered"

}

#uninstall function
uninstall_pegasus()
{

	msg "deleting DellOpenManage provider..."
	cimprovider -d -m DellOpenManageTunnelProviderModule >/dev/null 2>&1
	cimprovider -r -m DellOpenManageTunnelProviderModule >/dev/null 2>&1
        
 
#delete symbolic link
	msg "removing symbolic link..."
	PEG_LIB_PATH=`getLibpath`
	rm $PEG_LIB_PATH/libcmpiTunnelProvider.so >/dev/null 2>&1
	
#ensure all capabilities of omacim are cleaned out
#delete classes listed in /var/pegasus/repository/root#dcim#sysman/classes
	msg "removing classes listed in /var/pegasus/repository/root#dcim#sysman/classes..."
	ls "/var/pegasus/repository/root#dcim#sysman/classes" -h | grep -i DCIM > temp.txt
			#ls "/var/pegasus/repository/root#dcim#sysman/classes" -h | grep -i DCIM > temp.txt
	CLASSES=`cat temp.txt 2> /dev/null | cut -d '.' -f 1`
	for cls in $CLASSES
	do
	  wbemexec > /dev/null <<EOFWX
<?xml version="1.0" encoding="utf-8" ?>
<CIM CIMVERSION="2.0" DTDVERSION="2.0">
 <MESSAGE ID="4711" PROTOCOLVERSION="1.0">
  <SIMPLEREQ>
   <IMETHODCALL NAME="DeleteClass">
    <LOCALNAMESPACEPATH>
     <NAMESPACE NAME="root"></NAMESPACE>
     <NAMESPACE NAME="dcim"></NAMESPACE>
     <NAMESPACE NAME="sysman"></NAMESPACE>
    </LOCALNAMESPACEPATH>
    <IPARAMVALUE NAME="ClassName">
     <CLASSNAME NAME="$cls"/>
    </IPARAMVALUE>
   </IMETHODCALL>
  </SIMPLEREQ>
 </MESSAGE>
</CIM>
EOFWX
	done
	rm temp.txt
	#rm -rf "/var/pegasus/repository/root#dcim#sysman"

# delete DCIM classes in /var/pegasus/repository/root#interop/classes
	msg "removing DCIM classes listed in /var/pegasus/repository/root#interop/classes..."
	ls "/var/pegasus/repository/root#interop/classes" -h | grep -i DCIM > temp.txt
		#ls "/var/lib/Pegasus/repository/root#interop/classes" -h | grep -i DCIM > temp.txt
	CLASSES=`cat temp.txt 2> /dev/null | cut -d '.' -f 1`
	for cls in $CLASSES
	do
	  wbemexec > /dev/null <<EOFWX
<?xml version="1.0" encoding="utf-8" ?>
<CIM CIMVERSION="2.0" DTDVERSION="2.0">
 <MESSAGE ID="4711" PROTOCOLVERSION="1.0">
  <SIMPLEREQ>
   <IMETHODCALL NAME="DeleteClass">
    <LOCALNAMESPACEPATH>
     <NAMESPACE NAME="root"></NAMESPACE>
     <NAMESPACE NAME="interop"></NAMESPACE>
    </LOCALNAMESPACEPATH>
    <IPARAMVALUE NAME="ClassName">
     <CLASSNAME NAME="$cls"/>
    </IPARAMVALUE>
   </IMETHODCALL>
  </SIMPLEREQ>
 </MESSAGE>
</CIM>
EOFWX
	done
	rm temp.txt
	
#remove capabilities from /var/pegasus/repository/root#PG_Interop/instances
	msg "removing capabilities from /var/pegasus/repository/root#PG_InterOp/instances..."
	CAP_PATH=/var/pegasus/repository/root#PG_InterOp/instances
		#CAP_PATH=/var/lib/Pegasus/repository/root#PG_InterOp/instances
	
	cp $CAP_PATH/PG_Provider.idx $CAP_PATH/PG_Provider.idx.bk
	cat $CAP_PATH/PG_Provider.idx | grep -v DellOpenManageTunnelProviderModule > $CAP_PATH/temp.txt
	cp $CAP_PATH/temp.txt $CAP_PATH/PG_Provider.idx
	
	cp $CAP_PATH/PG_ProviderModule.idx $CAP_PATH/PG_ProviderModule.idx.bk
	cat $CAP_PATH/PG_ProviderModule.idx | grep -v DellOpenManageTunnelProviderModule > $CAP_PATH/temp.txt
	cp $CAP_PATH/temp.txt $CAP_PATH/PG_ProviderModule.idx


	rm -rf "/var/pegasus/repository/root#dcim#sysman"
	
#Delete DellOpenManage provider
	#msg "deleting DellOpenManage provider..."
	#cimprovider -r -m DellOpenManageTunnelProviderModule >/dev/null 2>&1
    /etc/init.d/pegasus restart
	 #/etc/init.d/tog-pegasus restart
	
	isTunnelAvailable
	if [ $? -eq 0 ]
	then
		msg "DellOpenManageTunnelProviderModule successfully removed."
	fi

	msg "Pegasus Uninstallation complete"	
}

pegasus_repository()
{
    for p in $PEGASUS_HOME /var/lib/Pegasus /var/lib/pegasus \
	/usr/local/var/lib/pegasus /var/local/lib/pegasus /var/opt/tog-pegasus /var/pegasus
    do
      if test -d $p/repository
      then
	  echo $p/$1
	  return 0
      fi
    done
    return 1
}

#install sfcb
install_sfcb()
{

	status_sfcb
        if [ $? -eq 0 ]; then
                start_flag=1
		stop_sfcb
        else
                start_flag=0
        fi

	
	
#create a symbolic link to libomacim32/64.so from the /usr/lib/cmpi directory

	SFCB_LIB_PATH=`getLibpath`
	ln -sf $INSTALLPATH/lib/libcmpiTunnelProvider.so $SFCB_LIB_PATH/libcmpiTunnelProvider.so 

#run sfcbstage command
    # Q - ensure that dcim_server terminates in mof - think abt this
	msg "running sfcbstage command..."
	if [ $OS -ne 8 ]; then
		sfcbstage -r $INSTALLPATH/etc/schema/tunnel.reg
	else
		# for ESX 4.0
		cat $INSTALLPATH/etc/schema/tunnel.reg > ./tmp/tunnel-providerRegister
		sfcbstage -r ./tmp/tunnel-providerRegister
		cp ./tmp/tunnel-providerRegister /var/lib/sfcb/registration/.
	fi
		

        sfcbstage -n root/dcim/sysman $INSTALLPATH/etc/schema/tp.mof 
	sfcbstage -n root/interop/ $INSTALLPATH/etc/schema/int.mof
	
	
#rebuild
	msg "rebuilding sfcb..."
	if [ $OS -ne 8 ]; then
		for Schema_Dir in /usr/share/sfcb/CIM /usr/local/share/sfcb/CIM none
        	do
			if test -x $Schema_Dir
		        then
				if [ -f $Schema_Dir/CIM_Schema.mof ]; then
					msg "CIM_Schema.mof is installed..Creating Backup"
					copy_dell_schema $Schema_Dir $INSTALLPATH/CIM.Dell.Bk
					
				else
					msg "CIM_Schema.mof not installed..!!"
					copy_dell_schema $Schema_Dir $INSTALLPATH/CIM.Dell.Bk
				fi
        			break;
        		fi
        	done
		if test $Schema_Dir = none
		then
			msg "could not locate schema directory..!!"
			if [ ! -d "/usr/share/sfcb/CIM" ]; then
				mkdir "/usr/share/sfcb/CIM"
			fi
			copy_dell_schema "/usr/share/sfcb/CIM" $INSTALLPATH/CIM.Dell.Bk
			
		fi
		sfcbrepos -f
		
		rm -rf $Schema_Dir/* > /dev/null 2>&1
		cp -rf $INSTALLPATH/CIM.Dell.Bk/* $Schema_Dir/. > /dev/null 2>&1
		rm -rf $INSTALLPATH/CIM.Dell.Bk > /dev/null 2>&1
		 
	else
		# for ESX 4.0
		sfcbrepos -f -c /var/lib/sfcb/schema
	fi
			
	
	msg "tunnel is registered"

	if [ $start_flag -eq 1 ]; then	
		#start sfcb.
		start_sfcb	
	fi
}

sfcb_reg_unreg_failed()
{
	sfcbunstage -n root/dcim/sysman `ls $INSTALLPATH/etc/schema/ | grep tp.mof`
        sfcbunstage -n root/interop `ls $INSTALLPATH/etc/schema/ | grep int.mof`

	if [ $OS -ne 8 ]; then
		sfcbunstage -r tunnel.reg
                
	else
		sfcbunstage -r tunnel-providerRegister
		rm -f /var/lib/sfcb/registration/tunnel-providerRegister
	fi
		
	exit 1
}

copy_dell_schema()
{
	
	if [ -d $1 ]; then
		if [ -d $2 ]; then
			rm -rf $2 > /dev/null 2>&1
		fi
		mkdir $2 > /dev/null 2>&1
		cp -rf $1/* $2/. > /dev/null 2>&1
		rm -rf $1/* > /dev/null 2>&1
		cp $INSTALLPATH/etc/schema/cimv217Experimental-MOFs.zip $1/. > /dev/null 2>&1
		cd $1 > /dev/null 2>&1
		unzip cimv217Experimental-MOFs.zip > /dev/null 2>&1
		rm -f cimv217Experimental-MOFs.zip > /dev/null 2>&1
		cp cimv217.mof CIM_Schema.mof > /dev/null 2>&1
		cd $INSTALLPATH/ > /dev/null 2>&1
		return 0
	else
		return 1	
	fi
}

uninstall_sfcb()
{
	status_sfcb
	if [ $? -eq 0 ]; then
		start_flag=1
	else
		start_flag=0
	fi
	
	stop_sfcb

	SFCB_LIB_PATH=`getLibpath`
	rm -f $SFCB_LIB_PATH/libcmpiTunnelProvider.so

	sfcbunstage -n root/dcim/sysman `ls $INSTALLPATH/etc/schema/ | grep tp.mof`
	sfcbunstage -n root/interop `ls $INSTALLPATH/etc/schema/ | grep int.mof`
	if [ $OS -ne 8 ]; then
		sfcbunstage -r tunnel.reg
		sfcbrepos -f
	else
		sfcbunstage -r tunnel-providerRegister
		rm -f /var/lib/sfcb/registration/tunnel-providerRegister
		sfcbrepos -f -c /var/lib/sfcb/schema/
	fi
	
	if [ -d $INSTALLPATH/CIM.Dell.Bk ]; then
		rm -rf /usr/share/sfcb/CIM/* > /dev/null 2>&1
		if [ -d /usr/share/sfcb/CIM ]; then
			cp -rf $INSTALLPATH/CIM.Dell.Bk/* /usr/share/sfcb/CIM/. > /dev/null 2>&1
		fi
		rm -rf $INSTALLPATH/CIM.Dell.Bk > /dev/null 2>&1
		
	fi

	if [ $start_flag -eq 1 ]; then
		start_sfcb
	fi

}
status_sfcb()
{
	#if running return 0, otherwise 1
	if ps -C sfcbd > /dev/null 2>&1
	then
		return 0
	else
		return 1
	fi
}
start_sfcb()
{
	for INITSCRIPT in /etc/init.d/sfcb /usr/local/etc/init.d/sfcb /etc/init.d/sfcbd-watchdog none
        do
          if test -x $INITSCRIPT
          then
              break;
          fi
        done

	msg "Starting sfcb.."
	if test $INITSCRIPT = none
        then
            sfcbd -d
        else
            $INITSCRIPT start
        fi

}


stop_sfcb()
{
        if ps -C sfcbd > /dev/null 2>&1
        then
                #running..
                for INITSCRIPT in /etc/init.d/sfcb /usr/local/etc/init.d/sfcb /etc/init.d/sfcbd-watchdog none
        	do
          		if test -x $INITSCRIPT
          		then
              			break;
          		fi
        	done
		
		msg "Stoping sfcb.."		
		if test $INITSCRIPT = none
        	then
			kill -9 `pidof sfcbd`
        	else
            		$INITSCRIPT stop
			if ps -C sfcbd > /dev/null 2>&1
			then
				kill -9 `pidof sfcbd`
			fi
        	fi

 
        else
		msg "sfcb not running."
	fi

}


### End of functions ###

### START OF MAIN CODE ###

install=0
uninstall=0

#check for arguments
	if [ -z "$1" ]
	then
		help=1;
	fi

while [ -n "$1" ]
do
  case $1 in
      -h) help=1; 
	  shift;
	  break;;
      -i) install=1
	  shift;;
      -p) pathname="$2"; 
	  shift 2;;
      -u) uninstall=1
	  shift;;
      --) shift;
	  break;;
      **) break;;
  esac
done

if [ "$help" = "1" ]
then
    usage
    echo -e "\t-h display help message"
    echo -e "\t-p add a pathname where ssa directory is located"
    echo -e "\t-i install SSA"
    echo -e "\t-u uninstall SSA"
    echo
    echo Use this command to install or uninstall SSA.
    echo Installation will compile the mofs, register the providers.
	echo "$TUNNEL_LOG file will be created in the directory where this script resides"
    exit 0
fi

#create a log file
#	msg "logging starts..."
	mkdir -p $TUNNEL_LOG_PATH
	echo "logging starts..." > $TUNNEL_LOG

#get absolute path to pathname
	CUR_PATH=`pwd` 
	cd ../
	cd "$pathname"
	INSTALLPATH=`pwd` 
	msg "INSTALLPATH=$INSTALLPATH"
	cd "$CUR_PATH"
	
#get current OS and arch. type
	getOS
	OS=$?
	msg "system info:OS=$OS"	
	
	getarchtype
	archtype=$?
	
	if [ $archtype -eq 1 ]; then
		msg "system info:Archtype=64bit"
	else
		msg "system info:Archtype=32bit"
	fi	


#run appropriate functions for each OS/arch. type
	isSfcbPegasusOS
	case $? in
	1) 
		if [ $install -eq 1 ]; then
			#installing pegasus
			msg "Installing pegasus..."
			install_pegasus

	
		elif [ $uninstall -eq 1 ]; then
			msg "Uninstalling pegasus..."
			uninstall_pegasus

	
		fi
	;;
	2) 
		if [ $install -eq 1 ]; then
			#Installing SFCB
			msg "Installing SFCB..."
			install_sfcb

	
		elif [ $uninstall -eq 1 ]; then
			msg "Uninstalling SFCB..."
			uninstall_sfcb
		
	
		fi

	;;
		*) msg "unsupported OS for SSA" 1;;
	esac	

	if [ $install -eq 1 ]; then
		echo "Installation complete." >> $TUNNEL_LOG
	elif [ $uninstall -eq 1 ]; then
		echo "Uninstallation complete." >> $TUNNEL_LOG
		rm -f $TUNNEL_LOG
	fi
	
	exit 0

